#include <math.h>
#define EPS 3.0e-14
#define MAXIT 10
#define N 6

double gammln(double xx);

void gaujac(double x[N], double w[N], double alf, double bet) {
	int i,its,j;
	double alfbet,an,bn,r1,r2,r3;
	double a,b,c,p1,p2,p3,pp,temp,z,z1,gl1,gl2,gl3,gl4;

	for (i=1;i<=N;++i) {
		if (i == 1) {
			an=alf/N;
			bn=bet/N;
			r1=(1.0+alf)*(2.78/(4.0+N*N)+0.768*an/N);
			r2=1.0+1.48*an+0.96*bn+0.452*an*an+0.83*an*bn;
			z=1.0-r1/r2;
		} else if (i == 2) {
			r1=(4.1+alf)/((1.0+alf)*(1.0+0.156*alf));
			r2=1.0+0.06*(N-8.0)*(1.0+0.12*alf)/N;
			r3=1.0+0.012*bet*(1.0+0.25*fabs(alf))/N;
			z -= (1.0-z)*r1*r2*r3;
		} else if (i == 3) {
			r1=(1.67+0.28*alf)/(1.0+0.37*alf);
			r2=1.0+0.22*(N-8.0)/N;
			r3=1.0+8.0*bet/((6.28+bet)*N*N);
			z -= (x[0]-z)*r1*r2*r3;
		} else if (i == N-1) {
			r1=(1.0+0.235*bet)/(0.766+0.119*bet);
			r2=1.0/(1.0+0.639*(N-4.0)/(1.0+0.71*(N-4.0)));
			r3=1.0/(1.0+20.0*alf/((7.5+alf)*N*N));
			z += (z-x[N-4])*r1*r2*r3;
		} else if (i == N) {
			r1=(1.0+0.37*bet)/(1.67+0.28*bet);
			r2=1.0/(1.0+0.22*(N-8.0)/N);
			r3=1.0/(1.0+8.0*alf/((6.28+alf)*N*N));
			z += (z-x[N-3])*r1*r2*r3;
		} else {
			z=3.0*x[i-2]-3.0*x[i-3]+x[i-4];
		}
		alfbet=alf+bet;
		for (its=1;its<=MAXIT;++its) { 
			temp=2.0+alfbet;
			p1=(alf-bet+temp*z)/2.0;
			p2=1.0;
			for (j=2;j<=N;++j) {
				p3=p2;
				p2=p1;
				temp=2*j+alfbet;
				a=2*j*(j+alfbet)*(temp-2.0);
				b=(temp-1.0)*(alf*alf-bet*bet+temp*(temp-2.0)*z);
				c=2.0*(j-1+alf)*(j-1+bet)*temp;
				p1=(b*p2-c*p3)/a;
			}
			pp=(N*(alf-bet-temp*z)*p1+2.0*(N+alf)*(N+bet)*p2)/(temp*(1.0-z*z));
			z1=z;
			z=z1-p1/pp;
			if (fabs(z-z1) <= EPS) break;
		}
		if (its > MAXIT) {
			printf("too many iterations in gaujac\n");
			exit(1);
		}
		x[i-1]=z;
		gl1 = gammln(alf+N);
		gl2 = gammln(bet+N);
		gl3 = gammln(N+1.0);
		gl4 = gammln(N+alfbet+1.0);
		w[i-1]=exp(gl1+gl2-gl3-gl4)*temp*pow(2.0,alfbet)/(pp*p2);
	}
}

double gammln(double xx) {
	double x,y,tmp,ser;
	static double cof[6]={76.18009172947146,-86.50532032941677,
												24.01409824083091,-1.231739572450155,
												0.1208650973866179e-2,-0.5395239384953e-5};
	int j;

	y=x=xx;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.000000000190015;
	for (j=0;j<=5;++j) ser += cof[j]/++y;
	return -tmp+log(2.5066282746310005*ser/x);
}
